﻿using System;
using System.Net;
using System.Threading;
using EventStore.ClientAPI;
using EventStore.ClientAPI.SystemData;

namespace EventStoreSubscriptionIssue
{
    class Program
    {
        static IEventStoreConnection _connection;

        static void Main(string[] args)
        {
            var sb = ConnectionSettings.Create();
            sb.SetDefaultUserCredentials(new UserCredentials("admin", "changeit"));
            _connection = EventStoreConnection.Create(sb.Build(), new IPEndPoint(IPAddress.Parse("127.0.0.1"), 1113));
            _connection.ConnectAsync().Wait();

            Subscribe();

            Console.WriteLine("Running. Press any key to exit...");
            Console.ReadKey(true);
        }
        
        private static void Subscribe()
        {
            Console.WriteLine("Subscribing to all events");
            _connection.SubscribeToAllFrom(new Position(76047864, 76047864), false, EventAppeared, LiveProcessingStarted, SubscriptionDropped);
        }

        private static void LiveProcessingStarted(EventStoreCatchUpSubscription obj)
        {
            Console.WriteLine($"{DateTime.UtcNow}: Live processing started on managed thread {Thread.CurrentThread.ManagedThreadId}");
        }

        // 76047864/76047864
        private static void EventAppeared(EventStoreCatchUpSubscription subscription, ResolvedEvent resolvedEvent)
        {
            Console.WriteLine($"{DateTime.UtcNow}: Received event {resolvedEvent.OriginalPosition} ({resolvedEvent.OriginalEvent.EventType}) on managed thread {Thread.CurrentThread.ManagedThreadId}");
        }

        private static void SubscriptionDropped(EventStoreCatchUpSubscription subscription, SubscriptionDropReason reason, Exception ex)
        {
            Console.WriteLine($"Subscription dropped with reason {reason}. Resubscribing...");
            Subscribe();
        }

    }
}
